// BoostC version : 6.95
// Single user Full License for 1 node
// Target         : PIC18F4420
// Adappted to use with the nRF24L01
// by Martin Hug 2012
//
// The transmitter UNIT scans the analog/ digital i/o of the PIC18F4420
// and send the data using the spi to the nRF124L01.
////////////////////////////////////////////////////////////////////////////

#include <system.h>
#include "serial.h"
#include "RN171.h"



#include <PIC18f4420.h>

#ifdef _PIC18
#pragma DATA _CONFIG1H, _OSC_INTIO67_1H     // Int OSC, Port RA6,RA7 I/O.
#pragma DATA _CONFIG2H, _WDT_OFF_2H       
#pragma DATA _CONFIG3H, _MCLRE_OFF_3H	
#endif _PIC18

#pragma CLOCK_FREQ 8000000                  // CLK 8Mhz.

#define PW_LED latd.5                       // PW LED.
#define PW_UP latd.4                        // 3V regulator on.
#define RESET latc.0                        // RN-171 reset.
#define GPIO9 lata.1                        // RN-171 toggle 5 times to factory reset (impulse >= 1s).

unsigned char count = 3, devider = 1;				// global variable used to separate cosm command.
unsigned char aux_flag = 0;									// global variable used in main function. 

void main()
{
	init();                                   // device and peripheral initialisation. 
	reset_array();														// make sure the array is filled with 0.  
	
	while (!rcif){														// waits until the connection is done. 
	serial_printf_cosm(99);			  						// 99 sends a string to controll the connection. 
	PW_LED_OnOff();														// toggle LED  Off. 
	delay_ms(500);
	}

	unsigned char get_byte;
  short adc;

   
  while (1){

   	if (tmr0if){														// every 1''
			while (count){ 
			count = scan_analog(count);		        // scan analog inputs and puts the results into a buffer.
			}
			count = 3;
			devider++;														// devider for result sum.
			PW_LED_OnOff();                       // toggle LED  Off.
			tmr0if = 0x00;												// clear overflow flag.
			}
		else if test_bit (aux_flag,0){
			serial_printf_cosm(devider);			  	// every 10'' data update to Cosm.
			clear_bit (aux_flag,0);
			reset_array();												// make sure araray is filled with 0.        
			devider = 1;
			}
	}
}
//
//

void init(){

  osccon = 01110000b;                       // internal oscillator at 8 MHz.
  while (!iofs);                            // iofs OSCCON stable bit.

  trisa = 00000111b;												// RA0 - RA1 analo in.
	trisc = 11000000b;                        // PORTC all out.
	portc = 00000001b;												// RC0 Hi, RC1-RC7 Low.
  trisd = 00000000b;                        // PORTD all out.
  portd = 00000000b;                         
	trise = 00000000b;                        // PORTE all out.
	porte = 00000000b;												// all low.

	adcon1 |= 0x0F;                           // All digital IO's, no analog ports required.
	
  adcon1 = 00001100b;                       // DEBUG RA0-RA2 analog, before RA0 - RA5 (00001001b) analog, rest digital I/O.
  adcon2 = 10100001b;                       // Result is right justified, conversion Ckl Fosc/8, TAD 8, 

	RESET = 1;															  // start RN-171.
	GPIO9 = 1;																// only to reset the unit.
	PW_UP = 1;																// 3V regulator on.

	// Serial interface
	serial_init(206);                         // 9600 baud; DEBUG before BRG = 103 (19200 baud at 8MHz).


	intcon = INTCON_CONF;                   	// 11100000b, GIE enabled, TMR0IE enabled.
  t0con = TIME0_CONF;                       // 10000001b, assigned to T0CON presc. 1/4 16 bit, enabled.
	t1con = TIME1_CONF;                       // 10110001b, 16 bit,T1CON, 1/8 presc. Timer1 enabled.
	pie1 = PIE1_CONF;													// 00000001 enable TMR1 overflow.
return;
}

void PW_LED_OnOff()
{
	PW_LED = ~PW_LED;                         // toggle PW_LED (connected to PORTD,5).
}