/*************************************************************
* Blocking serial interface code for the hardware UART
* of PIC16-range microcontrollers from Microchip
*
* (c) 2005, L. Hollevoet, boostc compiler.
*           Based on code from Yann Hendrikx.
*************************************************************/
#include <system.h>
#include "serial.h"
#include "adc_ng.h"

const char* cosm_connect =  "{$method$:$get$,$version$:$1.0.0$,$resource$ : $/feeds/73205$,$headers$ :{$X-ApiKey$:$y1Tc9_rcNW1844z_WCTr5TWUFAmSAKxlQ21QbHllVjZNYz0g$}}";
const char* cosm_part_0 = "{$method$: $put$,$resource$:"; 
const char* cosm_feed_id = "$/feeds/73205$,";	// M_DRUM feed dB, temp, LDR .
const char* cosm_part_1 = "$params$: {},$headers$: {$X-ApiKey$: $y1Tc9_rcNW1844z_WCTr5TWUFAmSAKxlQ21QbHllVjZNYz0g$},$body$: {$version$: $1.0.0$,";
const char* cosm_ids = "$datastreams$: [{$id$ : $dB$, $current_value$: $#$}, {$id$ : $temp$, $current_value$ : $#$}, {$id$: $lum$,  $current_value$ : $#$}]},";
const char* cosm_part_2 = "$token$: $0x12355$}";


unsigned char flag, array_pointer = 0;
char sum_devider; 
short test = 333, test_1 = 444, test_2 = 555;

////////////////////////////////////////////////////////////
// Hardware initialisation
// input argument: the baudrate scaler
////////////////////////////////////////////////////////////
void serial_init(char brg)
{
	// Enable USART function on PORTC (see PIC18F4620 datasheet page 201)
	trisc.7       = 1;			
	trisc.6       = 1; 		
	
	// Transmit control register
	txsta.TX9D    = 0;        // DEBUG before TX9
	txsta.TXEN    = 1;
	txsta.SYNC    = 0;
	txsta.BRGH    = 1;
	
	// Receive control register
	rcsta.SPEN    = 1;
	rcsta.RX9D    = 0;        // DEBUG before RX9
	rcsta.CREN    = 1;
	rcsta.ADDEN   = 0;
	
	// Baudcon register
	baudcon.BRG16 = 1;        // 16 bit Baud Rate Generator.
	baudcon.WUE   = 0;
	baudcon.ABDEN = 0;
	
	// Baud rate setting
    spbrg 	   = brg;       // brg can be usaed to write the baud rate.	
 }

////////////////////////////////////////////////////////////
// Print a single character to the serial port
// This funtion is skipped in debug mode since it will 
// block the debugger (the function waits for a hardware
// signal)
////////////////////////////////////////////////////////////
void serial_printf(char value)
{

char data;

#ifdef DBG_COMPILE
	// Skip when debugging
	return;
#endif
		flag = 0;
		if (value == 0x24)
		value = 0x22;
			if (value == 0x23){
				switch (array_pointer){
				case 0:
				serial_print_dec(data_convert(array_pointer));
				array_pointer++;
				break;
				case 1:
				serial_print_dec(data_convert(array_pointer));
				array_pointer++;
				break;
				case 2:
				serial_print_dec(data_convert(array_pointer));
				array_pointer++;
				break;
				}	
			}
			if (!flag){	
		   while((txsta & 1 << TRMT) == 0);
				txreg = value;
				}
}

//
//
////////////////////////////////////////////////////////////
// For Cosm dec. print

////////////////////////////////////////////////////////////
// Prints a line to the serial port. Ends with a linefeed.
// The function parameter is stored in rom.
////////////////////////////////////////////////////////////
void serial_printf(const char* text)
{
    char i = 0;
    while(text[i] != 0)
        serial_printf(text[i++]);
        
}

////////////////////////////////////////////////////////////
// Print a command to COSM
////////////////////////////////////////////////////////////
void serial_printf_cosm(char up){
	
		sum_devider = up; 
		char i = 0;

		if (sum_devider == 99){
		while(cosm_connect[i] != 0)
    serial_printf(cosm_part_0[i++]);
		i = 0;
		}

		while(cosm_part_0[i] != 0)
        serial_printf(cosm_part_0[i++]);
				i = 0;
	
		while(cosm_feed_id[i] != 0)										// feed id (73205) temp.
        serial_printf(cosm_feed_id[i++]);
				i = 0;


		while(cosm_part_1[i] != 0)
        serial_printf(cosm_part_1[i++]);
				i = 0;


		while(cosm_ids[i] != 0)
        serial_printf(cosm_ids[i++]);							// multiple ids (dB, temp LDR).
				i = 0;
		

		while(cosm_part_2[i] != 0)
        serial_printf(cosm_part_2[i++]);
		array_pointer = 0;

}


////////////////////////////////////////////////////////////
// Print a decimal value to the serial port
////////////////////////////////////////////////////////////
void serial_print_dec(char number)
{
    if(number > 99) {
        serial_printf(((number / 100) % 10) + '0');
    }
    if(number > 9) {
        serial_printf(((number / 10) % 10) + '0');
    }
    serial_printf((number % 10) + '0');
		flag = 1; // for Cosm.
}

////////////////////////////////////////////////////////////
// Print a decimal value to the serial port
// The number takes multiple characters. If a character 
// position is not used, it is filled with 0.
// Positions should be 1, 2 or 3.
////////////////////////////////////////////////////////////
void serial_print_dec(char number, char positions)
{

    if(number > 99) {
        serial_printf(((number / 100) % 10) + '0');
    } else {
		if (positions > 2){
			serial_printf('0');
		}
	}
    if(number > 9) {
        serial_printf(((number / 10) % 10) + '0');
    } else {
		if (positions > 1){
			serial_printf('0');
		}
	}
	serial_printf((number % 10) + '0');
}

////////////////////////////////////////////////////////////
// Print a decimal value to the serial port
////////////////////////////////////////////////////////////
void serial_print_dec(short number)
{
	short debug = number;

	if(number > 9999) {
        serial_printf(((number / 10000) % 10) + '0');
    }
	if(number > 999) {
        serial_printf(((number / 1000) % 10) + '0');
    }
    if(number > 99) {
        serial_printf(((number / 100) % 10) + '0');
    }
    if(number > 9) {
        serial_printf(((number / 10) % 10) + '0');
    }
    serial_printf((number % 10) + '0');
		flag = 1; // for Cosm.
}

////////////////////////////////////////////////////////////
// Print a hex value to the serial port
////////////////////////////////////////////////////////////
void serial_print_hex(char number)
{
    char hexChar;
    char i;
    for(i = 0; i < 2; i++)
    {
        if(i == 0)
            hexChar = number >> 4;
        else
            hexChar = number & 0x0F;
        if(hexChar < 10)
            hexChar = hexChar + '0';
        else
            hexChar = hexChar + ('A' - 10);
        serial_printf(hexChar);
    }
}

////////////////////////////////////////////////////////////
// Print a 16-bit hex value to the serial port
////////////////////////////////////////////////////////////
void serial_print_hex(short value){
	
    char value1 = (char)((value >> 8) & 0x00FF);
    
	serial_print_hex(value1);
 
    char value0 = (char)(value & 0x00FF);
    
	serial_print_hex(value0);
	
	return;
    
}
////////////////////////////////////////////////////////////
// Print a binary value to the serial port
////////////////////////////////////////////////////////////
void serial_print_bin(char number)
{
    char i;
    for(i = 0; i < 8; i++)
    {
		if (number & 10000000b){
			serial_printf('1');
		} else {
		    serial_printf('0');
		}
        number <<= 1;
    }
}

////////////////////////////////////////////////////////////
// Wait for a character to read it from the serial port
// This function is skipped when running in debug mode
// since else it blocks the simulator (waits for a 
// hardware signal from the UART)
////////////////////////////////////////////////////////////
char serial_getch(){

#ifdef DBG_COMPILE
	return 0x00;
#endif

	while (!serial_peek());
	return rcreg;
}

// Read a decimal value from the serial port
char serial_get_decimal(){
	char value = 0;
	char input = 0;

	input = serial_getch();
	
	while (input != 0x0D){ 
		serial_printf(input);
		value *= 10;
		value += (input-0x30);
		input = serial_getch();
	}
	serial_print_lf();
	return value;
}
//
//
short data_convert (unsigned char array){

	char pointer = array; 
	short result, aux;

		
		if (pointer == 0){
			aux = data_10bit[pointer]/sum_devider;		//dB.
			result = (aux/5) - 4;
		}	
		if (pointer == 1){
			aux = data_10bit[pointer]/sum_devider;		// temp.
			result = (aux/6) - 2;
		}	
		if (pointer == 2){
			aux = data_10bit[pointer]/sum_devider;
			result = (672-aux);												// lumen.
		}
	return  result;
}